'''
Password Manager- 0.2.8

--- Avanindra Chakroborty 
|-----Arghya Sarkar

- New Features:

Completely Renovated Workflow

FLOW:

1. RUN pm init TO INITIALISE YOUR PM WITH DEFAULT CONFIGURATION

2. RUN pm config -ps <password> OR pm config --password <password> TO SET YOUR OWN PASSWORD (CHANGING THE DEFAULT ONE WHICH IS FETCHER)

3. NEW ENCRYPTION COMMAND :- pm encrypt <directory> <message> <note(optional)>
----- NOTE IS LIKE AN ID TO YOUR PASSWORD. FOR EXAMPLE YOU ARE STORING A PASSWORD FOR GOOGLE ACCOUNT. USE pm encrypt <directory> <message> googleac
----- THIS ID/NOTE IS USED TO LOCATE YOUR PASSWORDS WHILE DECRYPTING IT
----- USE pm encrypt current <message> <note(optional)> TO WORK IN YOUR CURRENT DIRECTORY

4. A DATA FILE ON THE SAME LOCATION GETS CREATED WHICH IS HIDDEN. THE SINGLE DATA FILE STORES ALL PASSWORDS

5. NEW DECRYPTION COMMAND:- THERE ARE TWO DECRYPTION COMMANDS

pm decrypt <directory> -n <note> OR pm decrypt <directory> --note <note>

THE ABOVE COMMAND DECRYPTS A SPECIFIC NOTE/ID OF YOUR PASSWORD

pm decryptf <directory>

THE ABOVE COMMAND GIVES YOU THE WHOLE LIST OF SAVED PASSWORDS

6. CLEAR COMMAND: - pm clear <directory> CLEARS ALL THE STORED DATA FROM YOUR DATA FILE

7. PM IS NOW MORE SECURE AND YOUR DATA IS STORED IN A SECURE WAY SO HACKERS.... GOOD BYE :)
'''

from click import secho
import cryptocode
import click
import os
import pickle
from pyfiglet import Figlet
from click_help_colors import HelpColorsGroup, HelpColorsCommand
from random import randint
import random
# Defining BinarytoDecimal() function
'''
================================================================================================
================================================================================================
'''
      
 
'''
================================================================================================
================================================================================================
'''  
# MAIN ENCRYPTION LOGIC

'''
================================================================================================
================================================================================================
'''

SYMBOLS = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz1234567890'

MAX_KEY_SIZE = len(SYMBOLS)
def getKey():
    cck = pickle.load(open((os.path.join(os.path.expanduser("~"), ".pmcli", ".cck.dat")), 'rb'))
    cek = pickle.load(open((os.path.join(os.path.expanduser("~"), ".pmcli", ".key.dat")), 'rb'))
    key = 0
    while True:
        key = cck
        if (key >= 1 and key <= MAX_KEY_SIZE):
            return key
def getTranslatedMessage(mode,message , key):
    if mode [0] == 'd':
        key = -key
    translated = ''
            
    for symbol in message:
        symbolIndex = SYMBOLS.find(symbol)
        if symbolIndex == -1:
            translated += symbol
        else:
            symbolIndex += key
            if symbolIndex >= len(SYMBOLS):
                symbolIndex -= len(SYMBOLS)
            elif symbolIndex < 0:
                symbolIndex += len(SYMBOLS)
                    
            translated += SYMBOLS[symbolIndex]
    return translated

'''
================================================================================================
================================================================================================
'''

# BASE COMMAND

@click.group(
    cls=HelpColorsGroup, help_headers_color="yellow", help_options_color="cyan"
)
@click.version_option('0.2.8')
def main():
    """PM: Encrypt, decrypt and save your passwords"""
    pass

'''
================================================================================================
================================================================================================
'''

# ENCRYPTION LOGIC

@main.command('encrypt', help='Encrypt your message')
@click.argument('content', nargs=1)
@click.option('--note','-n', nargs=1)
def encrypt(content, note):
    cck = pickle.load(open((os.path.join(os.path.expanduser("~"), ".pmcli", ".cck.dat")), 'rb'))
    cek = pickle.load(open((os.path.join(os.path.expanduser("~"), ".pmcli", ".key.dat")), 'rb'))
    cce = getTranslatedMessage('e', content, cck )   
    msg = cryptocode.encrypt(cce, cek)
    file = open((os.path.join(os.path.expanduser("~"), ".pmcli", ".data_encr.dat")), 'a')
    file.write((msg+f'------------------ {note}'+'\n'))

    

'''
================================================================================================
================================================================================================
'''

# SPECIFIC DECRYPTION LOGIC
@main.command('decrypt', help='Decrypt any of your passwords')
@click.option('--note', '-n', nargs=1)
def decrypt(note):
    cck = pickle.load(open((os.path.join(os.path.expanduser("~"), ".pmcli", ".cck.dat")), 'rb'))
    cek = pickle.load(open((os.path.join(os.path.expanduser("~"), ".pmcli", ".key.dat")), 'rb'))
    inp = click.prompt(click.style('Enter your password', fg='blue'), confirmation_prompt=True)
    ops = pickle.load(open((os.path.join(os.path.expanduser("~"), ".pmcli", ".pmcli.dat")), 'rb'))
    ops = cryptocode.decrypt(ops, cek)
    if inp==ops:
        if note==None:
            note=''
        a = open((os.path.join(os.path.expanduser("~"), ".pmcli", ".data_encr.dat")), 'r')
        b = a.readlines()
        res=0
        for line in b:
            if note in line:
                line= line.replace('------------------ {note}', '')
                line =line.replace('\n', '')
                c = line
                
                dmsg = cryptocode.decrypt(c, cek)
                ccd = getTranslatedMessage('d', dmsg, cck) 
                click.clear()
                secho(ccd, fg='blue', bg= 'black')
                break
            else:
                res+=1
        if res==(len(b)):
            secho('Invalid Identity Key', fg='red', bg= 'black')

    else:
        click.secho('Invalid Password', fg='red', bg= 'black')

'''
================================================================================================
================================================================================================
'''

# Mass DECRYPTION LOGIC

@main.command('decryptf', help='Decrypt all your passwords')
def decrypt():
    cck = pickle.load(open((os.path.join(os.path.expanduser("~"), ".pmcli", ".cck.dat")), 'rb'))
    cek = pickle.load(open((os.path.join(os.path.expanduser("~"), ".pmcli", ".key.dat")), 'rb'))
    inp = click.prompt(click.style('Enter your password', fg='blue'), confirmation_prompt=True)
    ops = pickle.load(open((os.path.join(os.path.expanduser("~"), ".pmcli", ".pmcli.dat")), 'rb'))
    ops = cryptocode.decrypt(ops, cek)
    if inp==ops:
        a = open((os.path.join(os.path.expanduser("~"), ".pmcli", ".data_encr.dat")), 'r')
        b = a.readlines()
        cd=[]
        for line in b:
            cd.append(line)
        for c in cd:   

            c = line
            
            dmsg = cryptocode.decrypt(c, cek)
            ccd = getTranslatedMessage('d', dmsg, cck) 
            click.clear()
            secho(ccd, fg='blue', bg= 'black')
    else:
        secho('Invalid Password', fg='red', bg= 'black')

'''
================================================================================================
================================================================================================
'''

# CLEAR COMMAND

@main.command(help='Clear existing data')
def clear():
    a = open((os.path.join(os.path.expanduser("~"), ".pmcli", ".data_encr.dat")), 'w')
    b = a.write('')

'''
================================================================================================
================================================================================================
'''

# SETUP COMMANDS

@main.command('init', help='Initialize pmcli for you to get started with it')
def init():
    try:
        os.mkdir((os.path.join(os.path.expanduser("~"), ".pmcli")))
        defps = 'fetcher'
        key = random.choice(SYMBOLS)
        pickle.dump(key, open((os.path.join(os.path.expanduser("~"), ".pmcli", ".key.dat")), 'wb'))
        key = pickle.load(open((os.path.join(os.path.expanduser("~"), ".pmcli", ".key.dat")), 'rb'))
        defps = cryptocode.encrypt(defps, key)
        pickle.dump(defps, open((os.path.join(os.path.expanduser("~"), ".pmcli", ".pmcli.dat")), 'wb'))
        cck = randint(1,25)
        pickle.dump(cck, open((os.path.join(os.path.expanduser("~"), ".pmcli", ".cck.dat")), 'wb'))
    except FileExistsError as e:
        print('File already exists')
@main.command('config', help='Set your configuration for pm')
@click.option('--password', '-ps')
def config(password):
    cck = pickle.load(open((os.path.join(os.path.expanduser("~"), ".pmcli", ".cck.dat")), 'rb'))
    cek = pickle.load(open((os.path.join(os.path.expanduser("~"), ".pmcli", ".key.dat")), 'rb'))
    inp = click.prompt(click.style('Enter your old password', fg='blue'), confirmation_prompt=True)
    ops = pickle.load(open((os.path.join(os.path.expanduser("~"), ".pmcli", ".pmcli.dat")), 'rb'))
    ops = cryptocode.decrypt(ops, cek)
    if inp==ops:
        passcode = click.prompt(click.style('Enter new password', fg='blue'), confirmation_prompt=True)
        password= cryptocode.encrypt(passcode, cek)
        pickle.dump(password, open((os.path.join(os.path.expanduser("~"), ".pmcli", ".pmcli.dat")), 'wb'))
    else:
        click.secho('Access Denied!', fg='red', bg= 'black')


@main.command('info', help='Information about PMCLI')
def info():
    """Info About CLI """
    f = Figlet(font='standard')
    click.secho(f.renderText('PM'), fg='green')
    click.secho("pm: a simple CLI password manager",fg='cyan')
    click.secho("\n\nPasswords are meant to be safe, \n  So it's our duty to save them!",fg='cyan')
    click.secho("\n\nDeveloper: Avanindra Chakraborty", fg='green', bg= 'black')

'''
================================================================================================
================================================================================================
'''

if __name__ == "__main__":
    main()
    


# -*- coding: utf-8 -*-
from setuptools import setup

modules = \
['pmcli']
install_requires = \
['click-help-colors>=0.9,<0.10',
 'click>=8.0.1,<9.0.0',
 'cryptocode>=0.1,<0.2',
 'pyfiglet>=0.8.post1,<0.9']

entry_points = \
{'console_scripts': ['pm = pmcli:main']}

setup_kwargs = {
    'name': 'pmcli',
    'version': '0.2.8',
    'description': 'A Command Line Interface to encrypt, decrypt and save passwords',
    'long_description': "# PM- Password Manager\n## `pm` : A simple CLI for managing passwords\n\nHave important passwords to store? Use pmcli to encrypt your passwords into files and decrypt them whenever you want!\n\n## Install\n\n```\npip3 install pmcli \n```\n\n## Dependencies\n+ `click` \n+ `click_help_colors`\n+ `pyfiglet`\n+ `cryptocode`\n\n## Built with\n+ `Python 3.9.5` \n\n## Supported Platforms:\n\n+ Operating System = Cross-Platform\n\n## How to use\n\nOpen powershell for Windows or Terminal for Linux/Mac and  and type ```pm```\n\nIf this result comes, then you have successfully installed pmcli on your system\n\n```\n  PM: Encrypt, decrypt and save your passwords\n\nOptions:\n  --version  Show the version and exit.\n  --help     Show this message and exit.\n\nCommands:\n  clear     Clear existing data\n  config    Set your configuration for pm\n  decrypt   Decrypt any of your passwords\n  decryptf  Decrypt all your passwords\n  encrypt   Encrypt your message\n  info      Information about PMCLI\n  init      Initialize pmcli for you to get started with it\n```\n\nelse, try the above steps again!\n\n#### Setup\n\nFirst you need to setup pm for yourself\n\nProcedure:\n\n- Run `pm init` to initialize pm for your directory\n\nThere should be no output\n\n- Run `pm config -ps <password>` to set your password for PM. It will ask you for your OLD PASSWORD\n  - Now if you haven't configured your passowrd before, enter `fetcher` in the OLD PASSWORD INPUT which is the default password\n\n**NOTE: THIS STORES YOUR PASSWORD FOR PM AND DOES NOT INDICATE A PASSWORD FOR PASSWORD MANAGEMENT**\n\n- Now you are set up to use PM\n\n##### Encryption:\n\n```\npm encrypt {password} {note(optional)}\n```\n\nFor example:\n\n```\npm encrypt 'Welcome982' -n google\n```\nHere the password `Welcome982` will be stored in encrypted format and stored in your device.\n\nIn the above example, 'google' is an ID that gives the encrypted data and identity.\n\nYou can even use `--note` instead of `-n` to add an ID\n\nGiving an ID is completely optional, but highly recommended for every user. This helps you decrypt your messages easily\n\n##### Decryption\n\nThere are two methods of decrypting/obtaining your passwords\n\n###### SPECIFIC DECRYPTION\n\n```\npm decrypt -n {note}\n```\n\nFor example:\n\n```\npm decrypt -n google\n```\n\nThis gives you the stored password identified by `google` NOTE/ID.\n\n###### MASS DECRYPTION\n\n```\npm decryptf \n```\n\nThis gives you all of your stored passwords\n\n```\npm decryptf\n```\n\n**NOTE: EVERY DECRYPTION METHOD NEEDS YOUT PM PASSWORD, HENCE IF YOU HAVE NOT SETUP YOUR PM, DECRYPTION WONT WORK**\n\n##### CLEAR\n\n`pm clear` cleans the data from data file\n\n## Release Notes\n\n- **Current Release- 0.2.4 (Major Update)**\n\n### What's new?\n\n- Double Encryption makes passwords safe and secure!\n- A beautiful TUI applied\n- Faster Performance\n- The Files are encrypted, and stored in your device, hence it's so secure that even you can't access them without pm\n\n#### Developers\n- [Avanindra Chakraborty](https://github.com/AvanindraC)\n- [Arghya Sarkar](https://github.com/arghyagod-coder)\n- [Shravan Asati](https://github.com/Shravan-1908)\n\n\n### Developer Tools\n\n- [Visual Studio Code](https://github.com/microsoft/vscode)\n\n- [Python 3.9.5](https://python.org)\n\n- [Git](https://git-scm.com)\n\n- [Python Poetry](https://python-poetry.org/)\n\n## License\n\nLicense © 2021-Present Avanindra Chakraborty\n\nThis repository is licensed under the MIT license. See [LICENSE](https://github.com/arghyagod-coder/lola/master/LICENSE) for details.\n\n## Special Notes\n\n- Contribution is appreciated! Visit the contribution guide in [Contribution Guide](CONTRIBUTING.md)\n- If you see anything uncomfortable or not working, file an issue in [the issue page](https://github.com/arghyagod-coder/lola/issues). Issues aren't ignored by the developers\n- Thanks for seeing my project!",
    'author': 'Avanindra Chakraborty',
    'author_email': 'avanindra.d2.chakraborty@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/AvanindraC/PMCLI',
    'py_modules': modules,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
